@@ -6,13 +6,32 @@
 import argparse
 import logging
 import os
+import sys
 
 from six import PY2
 
 from ._version import __version__
 from .log import init_logging, set_nbdime_log_level
 from .gitfiles import is_gitref
-from .diffing.notebooks import set_notebook_diff_targets
+from .diffing.notebooks import set_notebook_diff_targets, set_notebook_diff_ignores
+from .config import get_defaults_for_argparse, entrypoint_configurables
+from .prettyprint import pretty_print_dict, PrettyPrintConfig
+
+
+class ConfigBackedParser(argparse.ArgumentParser):
+
+    def parse_args(self, args=None, namespace=None, entrypoint=None):
+        if entrypoint is None:
+            entrypoint = self.prog
+        try:
+            defs = get_defaults_for_argparse(entrypoint)
+            ignore = defs.pop('Ignore', None)
+            self.set_defaults(**defs)
+            if ignore:
+                set_notebook_diff_ignores(ignore)
+        except ValueError:
+            pass
+        return super(ConfigBackedParser, self).parse_args(args=args, namespace=namespace)
 
 
 class LogLevelAction(argparse.Action):
@@ -38,6 +57,22 @@ class SkipAction(argparse.Action):
         pass
 
 
+class ConfigHelpAction(argparse.Action):
+    def __init__(self, option_strings, dest, help=None):
+        super(ConfigHelpAction, self).__init__(
+            option_strings, dest, nargs=0, help=help)
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        header = entrypoint_configurables[parser.prog].__name__
+        pretty_print_dict(
+            {
+                header: get_defaults_for_argparse(parser.prog),
+            },
+            config=PrettyPrintConfig(out=sys.stderr)
+        )
+        sys.exit(1)
+
+
 class IgnorableAction(argparse.Action):
     """Adds the supplied positive options and negative/ignore version as well"""
 
@@ -106,11 +141,16 @@ def add_generic_args(parser):
         '--version',
         action="version",
         version="%(prog)s " + __version__)
+    parser.add_argument(
+        '--config',
+        help="list the valid config keys and their current effective values",
+        action=ConfigHelpAction,
+    )
     parser.add_argument(
         '--log-level',
         default='INFO',
         choices=('DEBUG', 'INFO', 'WARN', 'ERROR', 'CRITICAL'),
-        help="Set the log level by name.",
+        help="set the log level by name.",
         action=LogLevelAction,
     )
 
@@ -161,6 +201,13 @@ def add_web_args(parser, default_port=8888):
         default=None,
         type=str,
         help="specify the browser to use, to override the system default.")
+    parser.add_argument(
+        '--persist',
+        action="store_true",
+        default=False,
+        help="prevent server shutting down on remote close request (when these"
+             " would normally be supported)."
+    )
     parser.add_argument(
         '--ip',
         default='127.0.0.1',
@@ -232,13 +279,42 @@ def add_diff_cli_args(parser):
     )
 
 
+def add_filter_args(diff_parser):
+    """Adds configuration for git commands where filter use is flagged"""
+    # Ideally, we would want to apply the filter only if we knew
+    # the file was not from a blob. However, this is not possible:
+    # If remote file path is equal to repo file path, it implies
+    # that the hex of remote equals the hex of the file on disk.
+    # Two possible cases can cause this:
+    # 1) Diffing against working dir (or stage when entire file is staged)
+    # 2) Diffing against a blob (clean) that happens to have the same hex as
+    #    the (smudged) file in working tree.
+    # Condition 1 should have filter applied, 2 should not.
+    # We can learn something by comparing the remote hash to the hash of the
+    # file in HEAD.
+    # - If they are equal, we know that is cannot come from a diff
+    #   agains working tree (git would not see it as changed),
+    #   so it must be from a blob (clean). No filter.
+    # - If they differ, consider the setup:
+    #   git co A; git co B -- file.path; git reset A
+    #   + remote could be from a working-tree diff: git diff (smudged, apply filter).
+    #   + remote could be from a blob: git diff A B (clean, no filter).
+    #
+    # These are undistinguishable to us. Therefore, we will always
+    # apply the filter to the remote file if flag use_filter is set.
+    diff_parser.add_argument(
+        '--use-filter',
+        action='store_true', default=False,
+        help='apply any configured git filters on remote')
+
+
 def add_git_diff_driver_args(diff_parser):
     """Adds a set of 7 stanard git diff driver arguments:
         path old-file old-hex old-mode new-file new-hex new-mode [ rename-to ]
 
     Note: Only path, base and remote are added to parsed namespace
     """
-
+    add_filter_args(diff_parser)
     diff_parser.add_argument('path')
     diff_parser.add_argument('base', nargs='?', default=None)
     diff_parser.add_argument('base_sha1', nargs='?', default=None, action=SkipAction)
@@ -301,25 +377,25 @@ def add_merge_args(parser):
         '--merge-strategy',
         default="inline",
         choices=cli_conflict_strategies,
-        help="Specify the merge strategy to use.")
+        help="the merge strategy to use.")
     parser.add_argument(
         '--input-strategy',
         default=None,
         choices=cli_conflict_strategies_input,
-        help="Specify the merge strategy to use for inputs "
+        help="the merge strategy to use for inputs "
              "(overrides 'merge-strategy' for inputs).")
     parser.add_argument(
         '--output-strategy',
         default=None,
         choices=cli_conflict_strategies_output,
-        help="Specify the merge strategy to use for outputs "
+        help="the merge strategy to use for outputs "
              "(overrides 'merge-strategy' for outputs).")
     parser.add_argument(
         '--no-ignore-transients',
         dest='ignore_transients',
         action="store_false",
         default=True,
-        help="Disallow deletion of transient data such as outputs and "
+        help="disallow deletion of transient data such as outputs and "
              "execution counts in order to resolve conflicts.")
 
 
@@ -348,7 +424,10 @@ def args_for_server(arguments):
                 workdirectory='cwd',
                 base_url='base_url',
                 )
-    return {kmap[k]: v for k, v in vars(arguments).items() if k in kmap}
+    ret = {kmap[k]: v for k, v in vars(arguments).items() if k in kmap}
+    if 'persist' in arguments:
+        ret['closable'] = not arguments.persist
+    return ret
 
 
 def args_for_browse(arguments):
